// XmlObject in JavaScript

// XmlAttribute
function XmlAttribute(name, value)
{
  this.name = name;
  this.value = value;

  function GetXmlString()
  {
    if (this.name == 0 || this.name.length <= 0)
      return "";
    if (this.value.length <= 0)
      return this.name;
    return this.name + "=\"" + this.value + "\"";
  }
  this.GetXmlString = GetXmlString;
}

// XmlObject

function claimChildren()
{
  for (var i = 0; i < this.nestedObjects.length; i++)
  {
    this.nestedObjects[i].nestingLevel = this.nestingLevel + 1;
    this.nestedObjects[i].claimChildren();
  }
}

function addObject(xo)
{
  this.nestedObjects.push(xo);
  this.claimChildren();
  return xo;
}

function setAttribute(name, value)
{
  var attr = this.findAttribute(name);
  if (attr == 0)
  {
    var attr = new XmlAttribute(name, value);
    this.attributes.push(attr);
  }
  else
  {
    attr.value = value;
  }
  return attr;
}

function findAttribute(attrName)
{
  for (var i = 0; i < this.attributes.length; i++)
  {
    if (this.attributes[i].name.toLowerCase() == attrName.toLowerCase())
      return this.attributes[i];
  }
  return 0;
}

function getAttributeValue(attrName, defaultValue)
{
  var attr = this.findAttribute(attrName);
  if (attr == 0)
    return defaultValue;
  return attr.value;
}

function findObjects(tagname)
{
  var resultList = new Array();

  for (var i = 0; i < this.nestedObjects.length; i++)
  {
    if (this.nestedObjects[i].tag != 0)
    {
      if (this.nestedObjects[i].tag.toLowerCase() == tagname.toLowerCase())
        resultList.push(this.nestedObjects[i]);
    }
  }
  return resultList;
}

function getCaptionMatch(captionStr)
{
  for (var i = 0; i < this.nestedObjects.length; i++)
  {
    var thisCaption = this.nestedObjects[i].getAttributeValue("caption");
    if (thisCaption == captionStr)
      return i;
  }
  return -1;
}

function getXmlString()
{
  var result = "";
  var indentString = "";
  for (var i = 0; i < this.nestingLevel; i++)
  {
    indentString += "  ";
  }

  if (this.tag != 0 && this.tag.length > 0)
    result += (indentString + "<" + this.tag);

  for (var i = 0; i < this.attributes.length; i++)
  {
    result += (" " + this.attributes[i].GetXmlString());
  }

  if (this.tag != 0 && this.tag.length > 0 && this.nestedObjects.length == 0)
  {
    result += "/>\r\n";
  }
  else
  {
    if (this.tag != 0 && this.tag.length > 0)
      result += ">\r\n";
    if (this.content != 0 && this.content.length > 0)
      result += (this.content + "\r\n");
    for (var i = 0; i < this.nestedObjects.length; i++)
    {
      result += this.nestedObjects[i].getXmlString();
    }
    if (this.tag != 0 && this.tag.length > 0)
      result += (indentString + "</" + this.tag + ">\r\n");
  }
  return result;
}

function getDisplayableString()
{
  var result = "";
  var indentString = "";
  for (var i = 0; i < this.nestingLevel; i++)
  {
    indentString += "&nbsp;&nbsp;";
  }

  if (this.tag != 0 && this.tag.length > 0)
    result += (indentString + "&lt;" + this.tag);

  for (var i = 0; i < this.attributes.length; i++)
  {
    result += (" " + this.attributes[i].GetXmlString());
  }

  if (this.tag != 0 && this.tag.length > 0 && this.nestedObjects.length == 0)
  {
    result += "/&gt;\r\n";
  }
  else
  {
    if (this.tag != 0 && this.tag.length > 0)
      result += "&gt;\r\n";
    if (this.content != 0 && this.content.length > 0)
      result += (this.content + "\r\n");
    for (var i = 0; i < this.nestedObjects.length; i++)
    {
      result += this.nestedObjects[i].getDisplayableString();
    }
    if (this.tag != 0 && this.tag.length > 0)
      result += (indentString + "&lt;/" + this.tag + "&gt;\r\n");
  }
  return result;
}

function parse(xstr)
{
  // some useful regular expressions
  var firstNonWhiteSpace = /\S/;
  var leadingWhiteSpace = /^\s+/;
  var trailingWhiteSpace = /\s+$/;
  firstNonWhiteSpace.multiline = true;
  leadingWhiteSpace.multiline = true;
  trailingWhiteSpace.multiline = true;

  // Remove leading and trailing white-space.
  xstr.replace(leadingWhiteSpace, "");
  xstr.replace(trailingWhiteSpace, "");

  var bigRE = "<.*\n*>|[^\r\n<]+";

  var shallowTokens = xstr.match(new RegExp(bigRE, "g"));
  var objectStack = new Array();

  for (var i = 0; shallowTokens && i < shallowTokens.length; i++)
  {
    // Remove leading and trailing white-space.
    shallowTokens[i].replace(leadingWhiteSpace, "");
    shallowTokens[i].replace(trailingWhiteSpace, "");
    if (shallowTokens[i].length == 0)
      continue;
    if (shallowTokens[i].charAt(0) == '<' && shallowTokens[i].charAt(shallowTokens[i].length - 1) == '>')
    {
      var tagGuts = shallowTokens[i].slice(1, shallowTokens[i].length - 1);
      var tagExpression = /(\w+\s*=\s*\"[^\"]*\")|(\w+\s*=\s*\'[^\']*\')|(\w+\s*=\s*[^\"\' \t\n\r\f\v]+)|(\w+)/g;
      tagExpression.multiline = true;
      var deepTokens = tagGuts.match(tagExpression);
      if (tagGuts.charAt(0) == '/')
      {
        if (objectStack.length == 1)
          {
            var obj = objectStack.pop();
            this.tag = obj.tag;
            this.content = obj.content;
            this.attributes = obj.attributes;
            this.nestedObjects = obj.nestedObjects;
          }
        else if (objectStack.length > 1)
          {
            var obj = objectStack.pop();
            objectStack[objectStack.length - 1].addObject(obj);
          }
      }
      else
      {
        var obj = new XmlObject();
        obj.tag = deepTokens[0];
        for (var ii = 1; ii < deepTokens.length; ii++)
        {
          var attrExpression = /(\"[^\"]*\")|(\'[^\']*\')|([^= \t\n\r\f\v]+)/g;
          attrExpression.multiline = true;
          var attrTokens = deepTokens[ii].match(attrExpression);
          if (attrTokens == 0 || attrTokens.length < 1)
            continue;
          var name = attrTokens[0];
          var value = "";
          if (attrTokens.length == 2 && attrTokens[1].length > 0)
          {
            value = attrTokens[1];
            if (value.charAt(0) == '\'' || value.charAt(0) == '\"')
              value = value.substr(1);
            if (value.charAt(value.length-1) == '\'' || value.charAt(value.length-1) == '\"')
              value = value.substr(0, value.length-1);
          }
          obj.setAttribute(name, value);
        }
        if (tagGuts.charAt(tagGuts.length - 1) == '/' && objectStack.length > 0) // shortcut close-tag
          objectStack[objectStack.length - 1].addObject(obj);
        else
          objectStack.push(obj);
      }
    }
    else if (objectStack.length > 0)
    {
      var obj = new XmlObject();
      obj.content = shallowTokens[i];
      objectStack[objectStack.length - 1].addObject(obj);
    }
  }

  // Close any remaining (even though they ought to be already closed.
  while(objectStack.length > 0)
  {
    if (objectStack.length == 1)
      {
        var obj = objectStack.pop();
        this.tag = obj.tag;
        this.content = obj.content;
        this.attributes = obj.attributes;
        this.nestedObjects = obj.nestedObjects;
      }
    else if (objectStack.length > 1)
      {
        var obj = objectStack.pop();
        objectStack[objectStack.length - 1].addObject(obj);
      }
  }

  this.nestingLevel = 0;
  this.claimChildren();
}

// XmlObject Constructor
function XmlObject(xstr)
{
  this.tag = 0;
  this.content = 0;
  this.nestingLevel = 0;
  this.attributes = new Array();
  this.nestedObjects = new Array();
  this.claimChildren = claimChildren;
  this.addObject = addObject;
  this.setAttribute = setAttribute;
  this.findAttribute = findAttribute;
  this.getAttributeValue = getAttributeValue;
  this.findObjects = findObjects;
  this.getCaptionMatch = getCaptionMatch;
  this.getXmlString = getXmlString;
  this.getDisplayableString = getDisplayableString;
  this.parse = parse;

  if (typeof xstr == "string")
  {
    this.parse(xstr);
    this.claimChildren();
  }
}


