#!/bin/sh

# 
# link_diags.sh - Create a directory structure on the ramdrive that mirrors
#                 the diagnostic image on CD-ROM, and symbolically link all
#                 diag files from the CD-ROM to the ramdrive.  Also, insert
#                 the diag driver into the kernel and create a device for it.
#
# Usage: link_diags.sh [sourcedir [destdir]]
#
# Parameters
#
#   sourcedir - The source directory on CD-ROM from which diags are copied to
#               the ramdrive (default = /mnt/cdrom/compaq/diags)
#
#   destdir   - The target directory on the ramdrive to which diags are copied
#               (default = /diags)
#

# Check usage and collect parameters
if test "$#" = 0
then
  sourcedir="/mnt/cdrom/compaq/diags"
  destdir="/diags"
elif test "$#" = 1
then
  sourcedir="$1"
  destdir="/diags"
elif test "$#" = 2
then
  sourcedir="$1"
  destdir="$2"
else
  echo -e "\nUsage: link_diags.sh [sourcedir [destdir]]\n" >&2
  exit 1
fi

if test ! -d "$sourcedir"
then
  echo -e "\nlink_diags.sh: Directory $sourcedir not found\n" >&2
  exit 1
fi

mkdir -p $destdir
if test "$?" != 0
then
  echo -e "\nlink_diags.sh: Cannot create directory $destdir\n" >&2
  exit 1
fi

# Create diagnostic directory structure under destination directory
for dir in `cd $sourcedir; find * -type d -print`
do
  mkdir -p $destdir/$dir
done

# Symbolically link all files to the destination directory
for file in `cd $sourcedir; find * -type f -print`
do
  ln -s $sourcedir/$file $destdir/$file
done

#
# undo the symbolic link for diagconfig.xml and copy the actual file 
# into RAM disk, make sure its editable
#
# - this is for systems = 128MB RAM, want to edit which tcs get loaded
rm $destdir/diagconfig.xml
cp $sourcedir/diagconfig.xml $destdir
chmod +rw $destdir/diagconfig.xml



#
# link in allboards.xml for device information
#
cd $destdir
ln -s /mnt/cdrom/compaq/utils/allboards.xml
cd -

#for file in `cd $destdir/cpqhealth; find *.o -type f -print`
#do
#  insmod $destdir/cpqhealth/$file
#done

#
# load up parallel port drivers
#
modprobe parport_pc

#
# mount USB /proc filesystem
#
mount -t usbdevfs none /proc/bus/usb

#
# diag driver is re-enabled -- PCI BIOS problems have been resolved
#
insmod -f /diags/hpdvmkd.o
mknod /dev/hpdvmkd c 60 0

exit 0

