#!/bin/sh

# 
# pre_run_diags.sh - Performs all necessary functions that need to be done    
#                    before diagnostics is run.  Includes loading driver      
#                    modules and linking in appropriate files.                
#
# Usage: pre_run_diags.sh
#
# Parameters
#
#   none
#

# location of allboards.xml and where it needs to go
SOURCE="/mnt/cdrom/compaq/utils"
DEST="/opt/compaq/diags"

# load parallel port module if its not already
if test -z "`lsmod | grep parport_pc`"
then
  insmod parport_pc
  echo "parport_pc modprobed."
else
  echo "parport_pc already loaded."
fi

# mount the USB filesystem into /proc if its not there
if test -z "`ls /proc/bus/usb | grep devices`"
then
  mount -t usbdevfs none /proc/bus/usb
  echo "usbdevfs mounted."
else
  echo "usbdevfs already mounted."
fi

# link in allboards.xml for device information
if test -z "`ls $DEST | grep allboards.xml`"
then
  ln -s $SOURCE/allboards.xml $DEST 
  echo "allboards.xml linked."
else
  echo "allboards.xml already linked."
fi

# for McReady drive on 5i SmartArray
echo "engage scsi" > /proc/driver/cciss/cciss0


#
# from install_diags.sh - install the diagnostic kernel-mode driver
#

DRIVER="hpdvmkd.o"
DEVICE="/dev/hpdvmkd"
MAJORNUM=60

if test ! -f "$DRIVER"
then
  echo "$0: $DRIVER not found" >&2
  exit 1
elif test -n "`lsmod | grep hpdvmkd`"
then
  echo "$0: diagnostics are already installed on this system" >&2
  exit 1
fi

echo "installing diagnostics driver."
rm -f $DEVICE
insmod -f $DRIVER || exit 1
mknod $DEVICE c $MAJORNUM 0 || exit 1
